package com.tzld.longarticle.recommend.server;

import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticleCategoryStatusEnum;
import com.tzld.longarticle.recommend.server.mapper.aigc.PublishContentMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.ArticleAuditMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.ArticleCategoryMapper;
import com.tzld.longarticle.recommend.server.mapper.longArticle.LongArticleBaseMapper;
import com.tzld.longarticle.recommend.server.model.dto.PublishContentDTO;
import com.tzld.longarticle.recommend.server.model.dto.kimi.KimiResult;
import com.tzld.longarticle.recommend.server.model.entity.aigc.ProducePlan;
import com.tzld.longarticle.recommend.server.model.entity.aigc.ProducePlanExeRecord;
import com.tzld.longarticle.recommend.server.model.entity.aigc.PublishAccount;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.ArticleCategory;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.ArticleUnsafeTitle;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.IllegalArticles;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.LongArticleTitleAudit;
import com.tzld.longarticle.recommend.server.model.param.PublishContentParam;
import com.tzld.longarticle.recommend.server.remote.DeepSeekApiService;
import com.tzld.longarticle.recommend.server.repository.aigc.ProducePlanExeRecordRepository;
import com.tzld.longarticle.recommend.server.repository.aigc.ProducePlanRepository;
import com.tzld.longarticle.recommend.server.repository.aigc.PublishAccountRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.ArticleCategoryRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.ArticleUnsafeTitleRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.IllegalArticlesRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.LongArticleTitleAuditRepository;
import com.tzld.longarticle.recommend.server.util.TitleSimilarCheckUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.util.StringUtils;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@SpringBootTest(classes = Application.class)
@Slf4j
public class ArticleVideoAuditTest {

    @Resource
    private ArticleAuditMapper articleAuditMapper;
    @Resource
    private LongArticleTitleAuditRepository titleAuditRepository;
    @Resource
    private ProducePlanExeRecordRepository exeRecordRepository;
    @Resource
    private ProducePlanRepository producePlanRepository;
    @Resource
    private ArticleCategoryMapper articleCategoryMapper;
    @Autowired
    ArticleCategoryRepository articleCategoryRepository;
    @Autowired
    DeepSeekApiService deepSeekApiService;
    @Autowired
    ArticleUnsafeTitleRepository articleUnsafeTitleRepository;
    @Autowired
    IllegalArticlesRepository illegalArticlesRepository;
    @Autowired
    PublishAccountRepository publishAccountRepository;
    @Autowired
    PublishContentMapper publishContentMapper;
    @Autowired
    LongArticleBaseMapper longArticleBaseMapper;

    @Value("${kimiCategoryPrompt:}")
    private String kimiCategoryPrompt;

    @Test
    public void updateTitleAuditFlowPoolLevel() {
        List<LongArticleTitleAudit> list = titleAuditRepository.getByFlowPoolLevelIsNull();
        List<String> contentIds = list.stream().map(LongArticleTitleAudit::getContentId).collect(Collectors.toList());
        List<ProducePlanExeRecord> contentList = exeRecordRepository.findByPlanExeIdIn(contentIds);
        Map<String, String> contentPlanMap = contentList.stream().collect(Collectors.toMap(ProducePlanExeRecord::getPlanExeId, ProducePlanExeRecord::getPlanId));
        List<String> planIds = contentList.stream().map(ProducePlanExeRecord::getPlanId).collect(Collectors.toList());
        List<ProducePlan> planList = producePlanRepository.findByIdIn(planIds);
        Map<String, String> planMap = planList.stream().collect(Collectors.toMap(ProducePlan::getId, ProducePlan::getPlanTag));
        for (LongArticleTitleAudit titleAudit : list) {
            String contentId = titleAudit.getContentId();
            String planId = contentPlanMap.get(contentId);
            String tag = planMap.get(planId);
            if (!StringUtils.hasText(tag)) {
                continue;
            }
            titleAudit.setFlowPoolLevel(tag);
            titleAuditRepository.save(titleAudit);
        }
        articleAuditMapper.updateTitleAuditFlowPoolLevel();
    }

    @Test
    public void ArticleCategoryTest() {
        List<ArticleCategory> dealList = articleCategoryRepository.getByStatusAndVersion(
                ArticleCategoryStatusEnum.SUCCESS.getCode(), 1);
        dealList = dealList.subList(0, 200);
        List<List<ArticleCategory>> partitionList = Lists.partition(dealList, 50);
        JSONArray result = new JSONArray();
        for (List<ArticleCategory> partition : partitionList) {
            List<String> partitionTitles = partition.stream().map(ArticleCategory::getTitle).distinct().collect(Collectors.toList());
            String prompt = buildKimiPrompt(partitionTitles);
            KimiResult kimiResult = deepSeekApiService.requestOfficialApi(prompt, null, null, true);
            long now = System.currentTimeMillis();
            JSONObject obj = null;
            if (kimiResult.isSuccess()) {
                try {
                    obj = JSONObject.parseObject(kimiResult.getResponse().getChoices().get(0).getMessage().getContent());
                } catch (Exception e) {
                    log.error(kimiResult.getResponse().getChoices().get(0).getMessage().getContent());
                }
            }
            for (ArticleCategory articleCategory : partition) {
                JSONObject res = new JSONObject();
                res.put("title", articleCategory.getTitle());
                res.put("category", articleCategory.getCategory());
                articleCategory.setKimiResult(kimiResult.getResponseStr());
                articleCategory.setUpdateTimestamp(now);
                if (kimiResult.isSuccess() && Objects.nonNull(obj) && obj.containsKey(articleCategory.getTitle())) {
                    articleCategory.setCategory(obj.getString(articleCategory.getTitle()));
                    articleCategory.setStatus(ArticleCategoryStatusEnum.SUCCESS.getCode());
                } else {
                    if (Objects.nonNull(obj)) {
                        for (Map.Entry<String, Object> entry : obj.entrySet()) {
                            if (TitleSimilarCheckUtil.isSimilar(articleCategory.getTitle(), entry.getKey(),
                                    TitleSimilarCheckUtil.ARTICLE_PROMOTION_THRESHOLD)) {
                                articleCategory.setCategory(obj.getString(entry.getKey()));
                                articleCategory.setStatus(ArticleCategoryStatusEnum.SUCCESS.getCode());
                                break;
                            }
                        }
                    }
                }
                if (articleCategory.getStatus().equals(ArticleCategoryStatusEnum.SUCCESS.getCode())) {
                    res.put("newCategory", articleCategory.getCategory());
                }
                result.add(res);
            }
        }

        Workbook workbook = new XSSFWorkbook();
        Sheet sheet = workbook.createSheet("ExampleSheet");
        int rowNum = 0;
        // 创建标题行
        Row titleRow = sheet.createRow(rowNum);
        Cell titleCell = titleRow.createCell(0);
        titleCell.setCellValue("标题");
        titleCell = titleRow.createCell(1);
        titleCell.setCellValue("kimi品类");
        titleCell = titleRow.createCell(2);
        titleCell.setCellValue("deepseek品类");

        for (Object obj : result) {
            JSONObject jsonObject = (JSONObject) obj;
            String title = jsonObject.getString("title");
            String category = jsonObject.getString("category");
            String newCategory = jsonObject.getString("newCategory");
            rowNum++;
            Row row = sheet.createRow(rowNum);
            Cell cell = row.createCell(0);
            cell.setCellValue(title);
            cell = row.createCell(1);
            cell.setCellValue(category);
            cell = row.createCell(2);
            cell.setCellValue(newCategory);
        }

        try (FileOutputStream outputStream = new FileOutputStream("/Users/wangyunpeng/Downloads/category.xlsx")) {
            workbook.write(outputStream);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                workbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

    private String buildKimiPrompt(List<String> titleList) {
        StringBuilder prompt = new StringBuilder(kimiCategoryPrompt);
        prompt.append("\n");
        for (String title : titleList) {
            prompt.append(title).append("\n");
        }
        return prompt.toString();
    }

    @Test
    public void articleCategoryAdd() {
        List<ArticleCategory> oldList = articleCategoryRepository.getByStatusAndVersion(
                ArticleCategoryStatusEnum.SUCCESS.getCode(), 1);
        Map<String, ArticleCategory> oldMap = oldList.stream()
                .filter(o -> !o.getProduceContentId().startsWith("video"))
                .collect(Collectors.toMap(ArticleCategory::getProduceContentId, o -> o));
        List<String> oldIds = oldList.stream().map(ArticleCategory::getProduceContentId)
                .filter(o -> !o.startsWith("video"))
                .collect(Collectors.toList());
        List<ArticleCategory> newList = articleCategoryRepository.getByStatusAndVersion(
                ArticleCategoryStatusEnum.SUCCESS.getCode(), 2);
        List<String> newIds = newList.stream().map(ArticleCategory::getProduceContentId)
                .filter(o -> !o.startsWith("video"))
                .collect(Collectors.toList());
        List<ArticleCategory> saveList = new ArrayList<>();
        for (String oldId : oldIds) {
            if (newIds.contains(oldId)) {
                continue;
            }
            ArticleCategory articleCategory = oldMap.get(oldId);
            articleCategory.setVersion(2);
            articleCategory.setKimiResult(null);
            articleCategory.setId(null);
            articleCategory.setStatus(0);
            articleCategory.setCategory(null);
            articleCategory.setFailReason(null);
            articleCategory.setRetryTimes(null);
            articleCategory.setCreateTimestamp(System.currentTimeMillis());
            saveList.add(articleCategory);
        }

        if (CollectionUtils.isNotEmpty(saveList)) {
            articleCategoryMapper.batchInsertArticleCategory(saveList);
        }
    }

    @Test
    public void unsafeReAudit() {
        List<ArticleUnsafeTitle> unsafeTitleList = articleUnsafeTitleRepository.getByStatus(1);
        List<IllegalArticles> illegalArticlesList = illegalArticlesRepository.findAll();
        Map<String, List<String>> illegalMap = illegalArticlesList.stream()
                .collect(Collectors.groupingBy(IllegalArticles::getTitle, Collectors.mapping(IllegalArticles::getIllegalReason, Collectors.toList())));
        unsafeTitleList = unsafeTitleList.stream()
                .filter(o -> {
                    List<String> illegalReasons = illegalMap.get(o.getTitle());
                    if (CollectionUtils.isNotEmpty(illegalReasons)) {
                        for (String illegalReason : illegalReasons) {
                            if (!illegalReason.contains("导流")) {
                                return false;
                            }
                        }
                    }
                    return true;
                })
                .filter(o -> o.getCreateTimestamp() < 1737302400000L)
                .collect(Collectors.toList());
        List<String> ghIds = unsafeTitleList.stream().map(ArticleUnsafeTitle::getGhId).distinct().collect(Collectors.toList());
        // 获取发布账号
        List<PublishAccount> publishAccountList = publishAccountRepository.getAllByGhIdIn(ghIds);
        Map<String, PublishAccount> publishAccountMap = publishAccountList.stream().collect(Collectors.toMap(PublishAccount::getGhId, o -> o));
        List<PublishContentParam> publishContentParamList = unsafeTitleList.stream().map(article -> {
            PublishContentParam item = new PublishContentParam();
            item.setTitle(article.getTitle());
            PublishAccount account = publishAccountMap.get(article.getGhId());
            if (Objects.nonNull(account)) {
                item.setPublishAccountId(account.getId());
                return item;
            }
            return null;
        }).filter(Objects::nonNull).collect(Collectors.toList());
        List<PublishContentDTO> publishContents = new ArrayList<>();
        for (List<PublishContentParam> partitions : Lists.partition(publishContentParamList, 100)) {
            publishContents.addAll(publishContentMapper.getPublishContentByTitle(partitions));
        }
        for (PublishContentDTO publishContent : publishContents) {
            if (publishContent.getSourceType() != 1) {
                continue;
            }
            String sourceId = publishContent.getSourceId();
            LongArticleTitleAudit titleAudit = titleAuditRepository.getByContentId(sourceId);
            if (Objects.nonNull(titleAudit)) {
                if (titleAudit.getStatus() == 1) {
                    longArticleBaseMapper.updateUnsafeStatusByTitle(publishContent.getTitle(), 0);
                }
//                titleAudit.setStatus(0);
//                titleAuditRepository.save(titleAudit);
//            } else {
//                ProducePlanExeRecord exeRecord = exeRecordRepository.getByPlanExeId(sourceId);
//                ProducePlan producePlan = producePlanRepository.getById(exeRecord.getPlanId());
//                titleAudit = new LongArticleTitleAudit();
//                titleAudit.setContentId(sourceId);
//                titleAudit.setStatus(0);
//                titleAudit.setFlowPoolLevel(producePlan.getPlanTag());
//                titleAudit.setProducePlanId(producePlan.getId());
//                titleAudit.setCreateTimestamp(System.currentTimeMillis());
//                titleAuditRepository.save(titleAudit);
            }

        }
    }

}
