package com.tzld.longarticle.recommend.server;

import cn.hutool.core.collection.CollectionUtil;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.tzld.longarticle.recommend.server.common.enums.recommend.ArticleCategoryStatusEnum;
import com.tzld.longarticle.recommend.server.mapper.longArticle.ArticleCategoryMapper;
import com.tzld.longarticle.recommend.server.model.dto.LongArticlesMatchVideoResponse;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.ArticleCategory;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.DatastatScore;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.LongArticlesMatchVideo;
import com.tzld.longarticle.recommend.server.model.entity.longArticle.LongArticlesText;
import com.tzld.longarticle.recommend.server.model.vo.IntermediateIndicatorsExport;
import com.tzld.longarticle.recommend.server.model.vo.NewSortStrategyExport;
import com.tzld.longarticle.recommend.server.repository.longArticle.ArticleCategoryRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.DatastatScoreRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.LongArticlesMatchVideoRepository;
import com.tzld.longarticle.recommend.server.repository.longArticle.LongArticlesTextRepository;
import com.tzld.longarticle.recommend.server.service.recommend.DataDashboardService;
import com.tzld.longarticle.recommend.server.util.DateUtils;
import com.tzld.longarticle.recommend.server.util.feishu.FeiShu;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.junit.jupiter.api.Test;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.data.util.Pair;
import org.springframework.http.HttpHeaders;

import javax.annotation.Resource;
import java.io.FileOutputStream;
import java.io.IOException;
import java.util.*;
import java.util.stream.Collectors;

@SpringBootTest(classes = Application.class)
@Slf4j
public class DataDashboardTest {

    @Resource
    DataDashboardService dataDashboardService;
    @Resource
    DatastatScoreRepository datastatScoreRepository;
    @Resource
    ArticleCategoryRepository articleCategoryRepository;
    @Resource
    ArticleCategoryMapper articleCategoryMapper;
    @Resource
    LongArticlesTextRepository longArticlesTextRepository;
    @Resource
    LongArticlesMatchVideoRepository longArticlesMatchVideoRepository;

    @Test
    public void test() {
        List<IntermediateIndicatorsExport> result = dataDashboardService.intermediateIndicatorsData("20241225");
        log.info(JSONObject.toJSONString(result));
    }

    @Test
    public void newSortStrategyData() {
        List<NewSortStrategyExport> result = dataDashboardService.newSortStrategyData("20241222",
                "20241223", "9", 0);
        log.info(JSONObject.toJSONString(result));
    }

    @Test
    public void producePlanAuditExport() {
        List<String> dateStrList = DateUtils.getBeforeDays(null, null, 86);
        dataDashboardService.producePlanAuditExport(dateStrList);
    }

    @Test
    public void delFeishuSheet() {
        List<String> delDateStrList = DateUtils.getBeforeDays(null, null, 14);
        Pair<String, Integer> token = FeiShu.requestAccessToken();
        HttpHeaders httpHeaders = new HttpHeaders();
        httpHeaders.add("Authorization", "Bearer " + token.getFirst());

        dataDashboardService.delFeishuSheet(httpHeaders, "7d4e12", 2, delDateStrList);
    }

    @Test
    public void contentFunnelExport() {
        List<String> dateStrList = DateUtils.getBeforeDays(null, null, 15);
        dataDashboardService.contentFunnelExport(dateStrList);
    }

    @Test
    public void contentGroupFunnelExport() {
        List<String> dateStrList = DateUtils.getBeforeDays(null, null, 7);
        dataDashboardService.contentGroupFunnelExport(dateStrList);
    }

    @Test
    public void datastatScore() {
        List<DatastatScore> scoreList = datastatScoreRepository.getBySourceIdIsNotNull();
        List<ArticleCategory> categoryList = articleCategoryRepository.getByStatusAndVersion(ArticleCategoryStatusEnum.SUCCESS.getCode(), 2);
        Map<String, ArticleCategory> categoryMap = categoryList.stream().collect(Collectors.toMap(ArticleCategory::getProduceContentId, category -> category));
        Map<String, ArticleCategory> titleMap = categoryList.stream().collect(Collectors.toMap(ArticleCategory::getTitle, category -> category, (a, b) -> a));
        Map<String, List<DatastatScore>> scoreMap = scoreList.stream().collect(Collectors.groupingBy(DatastatScore::getTitle));
        for (Map.Entry<String, List<DatastatScore>> entry : scoreMap.entrySet()) {
            String title = entry.getKey();
            List<DatastatScore> list = entry.getValue();
            ArticleCategory category = null;
            for (DatastatScore score : list) {
                category = categoryMap.get(score.getSourceId());
                if (Objects.nonNull(category)) {
                    category = titleMap.get(score.getTitle());
                    break;
                }
            }
            if (Objects.isNull(category)) {
                category = titleMap.get(title);
            }
            if (Objects.isNull(category)) {
                log.error("category not exists");
                continue;
            }
            articleCategoryMapper.updateArticleCategory(title, category.getCategory());
        }
    }

    @Test
    public void topArticleData() {
        List<String> contentIds = Arrays.asList("20250225140910894349738","20250225140931802158259","20250225140955934920660","20250225141009384910525","20250225141018247767730","20250225141038500284697","20250225142241458681814","20250225143045896388743","20250225143048989841153","20250225143430205895384","20250225143539155869333","20250225143851845615224","20250225144101438336566","20250225144651578273347","20250225144944338773551","20250225145410177228690","20250225145610166427856","20250303074635710564866","20250306054814994527883","20250306054825050632302","20250306054833595937287","20250306054842247280278","20250306054850727555147","20250306054908034204377","20250306054928475412517","20250306054938403126107","20250306054947529718363","20250306054956650264904","20250306055008746915465","20250306055019652653644","20250306055029174529932","20250306055038439252451","20250306055047767839152","20250306055051645101476","20250306055056541638450","20250306055101185668998","20250306055537341880785","20250306055657945218943","20250306055738753758983","20250306055840053264679","20250306055842314796022","20250306055853829880930","20250306055916559752164","20250306055936889849992","20250306061133900915150","20250306061145222480819","20250306061154079990216","20250306061154273985471","20250306061206999682543","20250306061207114410181","20250306061209153743280","20250306061213568137581","20250306061221054349048","20250306061221387731240","20250306061222100928299","20250306061223179702177","20250306061226418794700","20250306061237434128790","20250306061238984155401","20250306061239135467650","20250306061240480345135","20250306061249066104302","20250306061251016984995","20250306061259527261733","20250306063146613476806","20250306063237742882973","20250306063256118468457","20250306063304715386983","20250306063313989746619","20250306063451252511987","20250306063500713437784","20250306063501988187543","20250306063521895172549","20250306063546941518880","20250306063649953715835","20250306063710554262086","20250306063718965432179","20250306063808045651304","20250306063835959433160","20250306063844677731805","20250306063854068128002","20250306064042217695911","20250306064050818829930","20250306064059162533418","20250331060451939274335","20250331060504657790284","20250331060527323102739","20250331060537922304936","20250331060559269431333","20250331060612140573568","20250331060622320738255","20250331060639153917967","20250331060641040543439","20250331060653531309749","20250331062448120771252","20250331062815154600420","20250331062826500634682","20250331063214358828087","20250331063254823245968");
        List<LongArticlesText> articlesTextList = longArticlesTextRepository.getByContentIdIn(contentIds);
        List<String> titles = articlesTextList.stream().map(LongArticlesText::getArticleTitle).collect(Collectors.toList());
        List<LongArticlesMatchVideo> matchVideoList = longArticlesMatchVideoRepository.getByContentIdIn(contentIds);
        matchVideoList = matchVideoList.stream().filter(matchVideo -> matchVideo.getContentStatus() == 4).collect(Collectors.toList());
        Map<String, List<LongArticlesMatchVideo>> matchVideoMap = matchVideoList.stream().collect(Collectors.groupingBy(LongArticlesMatchVideo::getContentId));
        List<ArticleCategory> articleCategoryList = articleCategoryRepository.getByTitleInAndVersion(titles, 1);
        Map<String, String> categoryMap = articleCategoryList.stream().collect(Collectors.toMap(ArticleCategory::getTitle, ArticleCategory::getCategory, (a, b) -> a));
        List<JSONObject> result = new ArrayList<>();
        for (LongArticlesText articlesText : articlesTextList) {
            String contentId = articlesText.getContentId();
            String title = articlesText.getArticleTitle();
            String category = categoryMap.get(articlesText.getArticleTitle());
            JSONObject item = new JSONObject();
            item.put("id", contentId);
            item.put("title", title);
            item.put("category", category);
            result.add(item);
        }


        Workbook workbook = new XSSFWorkbook();
        Sheet sheet = workbook.createSheet("sheet1");
        int rowNum = 0;
        // 创建标题行
        Row titleRow = sheet.createRow(rowNum);
        Cell titleCell = titleRow.createCell(0);
        titleCell.setCellValue("id");
        titleCell = titleRow.createCell(1);
        titleCell.setCellValue("标题");
        titleCell = titleRow.createCell(2);
        titleCell.setCellValue("品类");
        for (JSONObject item : result) {
            rowNum++;
            Row row = sheet.createRow(rowNum);
            Cell cell = row.createCell(0);
            String id = item.getString("id");
            cell.setCellValue(id);
            cell = row.createCell(1);
            cell.setCellValue(item.getString("title"));
            cell = row.createCell(2);
            cell.setCellValue(item.getString("category"));
            List<LongArticlesMatchVideo> matchVideos = matchVideoMap.get(id);
            if (Objects.isNull(matchVideos)) {
                continue;
            }
            LongArticlesMatchVideo lastMatchVideo = null;
            for (LongArticlesMatchVideo matchVideo : matchVideos) {
                if (Objects.isNull(lastMatchVideo) || lastMatchVideo.getRequestTimestamp() < matchVideo.getRequestTimestamp()) {
                    List<LongArticlesMatchVideoResponse> responseList = JSONArray.parseArray(matchVideo.getResponse()
                            , LongArticlesMatchVideoResponse.class);
                    if (CollectionUtil.isNotEmpty(responseList) && Objects.nonNull(responseList.get(0).getVideoID())) {
                        lastMatchVideo = matchVideo;
                    }
                }
            }
            int i = 3;
            if (Objects.nonNull(lastMatchVideo)) {
                List<LongArticlesMatchVideoResponse> responseList = JSONArray.parseArray(lastMatchVideo.getResponse()
                        , LongArticlesMatchVideoResponse.class);
                for (LongArticlesMatchVideoResponse response : responseList) {
                    cell = row.createCell(i++);
                    cell.setCellValue(response.getVideoID());
                }
            }
        }
        try (FileOutputStream outputStream = new FileOutputStream("/Users/wangyunpeng/Downloads/top100.xlsx")) {
            workbook.write(outputStream);
        } catch (IOException e) {
            e.printStackTrace();
        } finally {
            try {
                workbook.close();
            } catch (IOException e) {
                e.printStackTrace();
            }
        }
    }

}
